/////////////////////////////////////////////////////////////////////////////
// Name:        wxmainapp.cpp
// Author:      XX
// Created:     XX/XX/XX
// Copyright:   
/////////////////////////////////////////////////////////////////////////////

// Include private headers
#include "wxmainapp.h"

#ifndef _WIN32
    #include <fcntl.h>
    #include <signal.h>
    #include <stdlib.h>

    extern void wxEntryCleanup();
#endif

// WDR: class implementations

//------------------------------------------------------------------------------
// MyFrame
//------------------------------------------------------------------------------

// WDR: event table for MyFrame

BEGIN_EVENT_TABLE(MyFrame,wxFrame)
    EVT_MENU(ID_QUIT, MyFrame::OnQuit)
    EVT_CLOSE(MyFrame::OnCloseWindow)
END_EVENT_TABLE()

MyFrame::MyFrame( wxWindow *parent, wxWindowID id, const wxString &title,
    const wxPoint &position, const wxSize& size, long style ) :
    wxFrame( parent, id, title, position, size, style )
{

}

// WDR: handler implementations for MyFrame

void MyFrame::OnQuit( wxCommandEvent &event )
{
     Close( TRUE );
}

void MyFrame::OnCloseWindow( wxCloseEvent &event )
{
    // if ! saved changes -> return
    Destroy();
#ifdef _WIN32
    ::PostQuitMessage(0);
#endif
}

//------------------------------------------------------------------------------
// MyApp
//------------------------------------------------------------------------------


IMPLEMENT_APP(MyApp)


MyApp::MyApp()
{
#ifdef _WIN32
    m_pServiceStarter = NULL;
#endif
    m_pLogfile = NULL;
    m_pPCSCManager = NULL;
    m_pApplicationManager = NULL;
    m_pCardChangeMonitor = NULL;
    m_pCRLScheduler = NULL;
    m_pDefaultHandler = NULL;
    m_pSCardHandler = NULL;
    m_bHibernate = TRUE;
}


bool MyApp::OnInit()
{
#ifdef _DEBUG
    MyFrame *frame = new MyFrame( NULL, -1, wxT("Belpic PC/SC Service"), wxPoint(20,20), wxSize(500,100) );
    frame->Show( TRUE );
#else
    #ifdef _WIN32
        MyFrame *frame = new MyFrame( NULL, -1, wxT("Belpic PC/SC Service"), wxPoint(20,20), wxSize(500,100) );
		// Start server
        BOOL bNT = TRUE;
        wxDynamicLibrary serviceLoader("Advapi32");
        if(serviceLoader.IsLoaded())
        {
            void *psrv = (void*)::GetProcAddress( serviceLoader.GetLibHandle(), "StartServiceCtrlDispatcherA" );
            if(psrv == NULL)
            {
                bNT = FALSE;
            }
            psrv = (void*)::GetProcAddress( serviceLoader.GetLibHandle(), "RegisterServiceCtrlHandlerExA" );
            if(psrv == NULL)
            {
                m_bHibernate = FALSE;
            }
            serviceLoader.Unload();
        }

        if(bNT)
        {
		    // Start the service controller handler
		    m_pServiceStarter = new ServiceStarter();
		    m_pServiceStarter->Start();
        }
        else
        {
            //MyFrame *frame = new MyFrame( NULL, -1, wxT("Belpic PC/SC Service"), wxPoint(20,20), wxSize(500,100) );
            typedef DWORD (WINAPI *TRegisterServiceProcess)(DWORD,DWORD);
            TRegisterServiceProcess pReg;
            wxDynamicLibrary win9XLoader("kernel32");
            if(win9XLoader.IsLoaded())
            {
                pReg = (TRegisterServiceProcess)::GetProcAddress( win9XLoader.GetLibHandle(), "RegisterServiceProcess");
                if(pReg != NULL)
                {
                    pReg(0,1);
                }
                win9XLoader.Unload();
            }
        }
    #else
            if (daemon(0, 0) < 0)
                return FALSE;

	        signal(SIGQUIT, SignalHandler);
	        signal(SIGTERM, SignalHandler);
	        signal(SIGINT, SignalHandler);
	        signal(SIGHUP, SignalHandler);
        
            atexit(AtExit);

    #endif
#endif

    MainStart();

    return TRUE;
}

int MyApp::OnExit()
{
    MainStop();
    return 0;
}

#ifndef _WIN32
void MyApp::AtExit()
{
    wxGetApp().OnExit();
    wxEntryCleanup();
}

void MyApp::SignalHandler(int iSigNr)
{
    AtExit();
}

int MyApp::Daemon()
{
    switch (fork())
    {
        case 0:  break;
        case -1: return -1;
        default: _exit(0);          /* exit the original process */
    }

    if (setsid() < 0)               /* shoudn't fail */
      return -1;

    chdir("/");

    umask(0);

    return 0;
}
#endif

void MyApp::MainStart()
{
		DWORD dwCheckPoint = 0;
   		CObjectManager *pObjectManager = PASS_GET_OBJECTMANAGER;
        m_pDefaultHandler = new CDefaultHandler();
        m_pSCardHandler = new CSCardHandler();
        pObjectManager->RegisterObject(m_pDefaultHandler);
        pObjectManager->RegisterObject(m_pSCardHandler);

		SetServiceProgress(SERVICE_START_PENDING, ++dwCheckPoint);

        // Read config file
        CConfig oConfig;
        oConfig.Load();
        unsigned long ulServerPort = oConfig.GetServerPort();
        BOOL bLogMode = oConfig.GetLogMode();
        const char *pLogFile = oConfig.GetLogFile(); 

        ULONG ulLogMask = LOGMASK_BELPIC_ERROR;
#ifdef _DEBUG
        ulLogMask = LOGMASK_FULL;
#endif

        CLoggerManager *pLogManager = new CLoggerManager();
        if(pLogFile != NULL)
        {
            m_pLogfile = new CLoggerManagerModule_File();
		    m_pLogfile->SetFileName ((char *)pLogFile);
		    m_pLogfile->OpenFile();
		    pLogManager->AddModule (m_pLogfile);
		    if (bLogMode == (int)TRUE)
		    {
			    pLogManager->EnableModule ("File");
		    }
        }
		pLogManager->SetLogLevel (ulLogMask);

		pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL, "************************************************************");
		pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL, "**************** Belpic PC/SC Service 1.0 ****************");
		pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL, " Starting ...");

        SetServiceProgress(SERVICE_START_PENDING, ++dwCheckPoint);

		CConnectionManager *pConnectionManager = new CConnectionManager();
		pConnectionManager->SetConnectionType ("Mono");
		pConnectionManager->SetServerBasePort (ulServerPort);
		pConnectionManager->SetMaxServerConnection (30);
		pConnectionManager->StartServer ();
		pConnectionManager->Start();

		SetServiceProgress(SERVICE_START_PENDING, ++dwCheckPoint);

        m_pPCSCManager = new CPCSCManager();
        m_pPCSCManager->Initialize();
        m_pApplicationManager = new CApplicationManager();
        m_pCardChangeMonitor = new CCardChangeMonitor();
        m_pCardChangeMonitor->SetPCSCManager(m_pPCSCManager);
        m_pCardChangeMonitor->Start();
        
		SetServiceProgress(SERVICE_START_PENDING, ++dwCheckPoint);

        if(oConfig.GetCRLSchedule())
        {
            m_pCRLScheduler = new CCRLScheduler();
            m_pCRLScheduler->Start();
        }

        SetServiceProgress(SERVICE_START_PENDING, ++dwCheckPoint);

		pLogManager->Log (__FILE__,__LINE__, LOGMASK_FULL, " Belpic PC/SC Service Started");
		pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL, "************************************************************");
		
        SetServiceProgress(SERVICE_RUNNING, 0);
}

void MyApp::MainStop()
{
    DWORD dwCheckPoint = 0;
    ULONG ulCounter;
    ulCounter = 0;

    SetServiceProgress(SERVICE_STOP_PENDING, ++dwCheckPoint);

    CMasterManager *pMasterManager = CMasterManager::Instance();
	CObjectManager *pObjectManager = PASS_GET_OBJECTMANAGER;

    CLoggerManager *pLogManager = (CLoggerManager *)pMasterManager->GetManager("Logger");
    if(pLogManager != NULL)
    {
	    pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL,"************************************************************");
	    pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL," Stopping ...");
    }

    if(m_pCRLScheduler != NULL)
    {
	    m_pCRLScheduler->Stop();

	    while (m_pCRLScheduler->CanBeDeleted() == FALSE && ulCounter < 500)
	    {
		    PASS_MILLISLEEP (10);
		    ulCounter++;
	    }

	    if (ulCounter == 500)
	    {
		    // Reached timeout
		    // Kill handler
		    m_pCRLScheduler->Kill();
	    }

	    delete m_pCRLScheduler;
	    m_pCRLScheduler = NULL;
    }   

    SetServiceProgress(SERVICE_STOP_PENDING, ++dwCheckPoint);

    ulCounter = 0;
    if(m_pCardChangeMonitor != NULL)
    {
	    m_pCardChangeMonitor->Stop();

	    while (m_pCardChangeMonitor->CanBeDeleted() == FALSE && ulCounter < 500)
	    {
		    PASS_MILLISLEEP (10);
		    ulCounter++;
	    }

	    if (ulCounter == 500)
	    {
		    // Reached timeout
		    // Kill handler
		    m_pCardChangeMonitor->Kill();
	    }

	    delete m_pCardChangeMonitor;
	    m_pCardChangeMonitor = NULL;
    }

    SetServiceProgress(SERVICE_STOP_PENDING, ++dwCheckPoint);

    if(m_pApplicationManager != NULL)
    {
        delete m_pApplicationManager;
        m_pApplicationManager = NULL;
    }

    if(m_pPCSCManager != NULL)
    {
        delete m_pPCSCManager;
        m_pPCSCManager = NULL;
    }

    ulCounter = 0;
    CConnectionManager *pConnectionManager = (CConnectionManager *)pMasterManager->GetManager("Connection");
    if(pConnectionManager != NULL)
    {
	    pConnectionManager->Stop();

	    while (pConnectionManager->CanBeDeleted() == FALSE && ulCounter < 500)
	    {
		    PASS_MILLISLEEP (10);
		    ulCounter++;
	    }

	    if (ulCounter == 500)
	    {
		    // Reached timeout
		    // Kill handler
		    pConnectionManager->Kill();
	    }

	    delete pConnectionManager;
	    pConnectionManager = NULL;
    }

    SetServiceProgress(SERVICE_STOP_PENDING, ++dwCheckPoint);

    if(pLogManager != NULL)
    {
	    pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL," Belpic PC/SC Service Stopped");
	    pLogManager->Log (__FILE__, __LINE__, LOGMASK_FULL,"************************************************************");

	    delete pLogManager;
	    pLogManager = NULL;

        if (m_pLogfile != NULL)
	    {
		    delete m_pLogfile;
		    m_pLogfile = NULL;
	    }
    }

    delete m_pDefaultHandler;
    m_pDefaultHandler = NULL;
    delete m_pSCardHandler;
    m_pSCardHandler = NULL;

    PASS_MILLISLEEP(100);
    pMasterManager->RemoveAll();
    pObjectManager->RemoveAll();
 
    SetServiceProgress(SERVICE_STOPPED, 0);
}

void MyApp::SetServiceProgress(DWORD dwStatus, DWORD dwCheckPoint)
{
#ifdef _WIN32
	if(m_pServiceStarter != NULL)
	{
		m_pServiceStarter->ServiceSetStatus(dwStatus, dwCheckPoint);
	}
#endif
}
