// EIDlib.cpp : Implementation of CEIDlib

#include "stdafx.h"
#include "EIDLibCtrl.h"
#include "EIDlib.h"
#include "MapCollection.h"
#include "ComSafeArray.h"

/////////////////////////////////////////////////////////////////////////////
// CEIDlib

CEIDlib::CEIDlib()
{
    m_lCardHandle = NULL;
}

CEIDlib::~CEIDlib()
{
    CComPtr <IRetStatus> pIRetStatus;
    if (NULL != m_lCardHandle)
    {
        m_lCardHandle = NULL;
        Exit (&pIRetStatus);
    }
}


STDMETHODIMP CEIDlib::Init(BSTR strReaderName, long lOCSP, long lCRL, long *plHandle, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    BEID_Status tStatus = {0};
    HRESULT hRes = S_OK;

    if(NULL == plHandle || ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Create a local return status object
    if (SUCCEEDED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
    {
        // Initialise the eidlib
        _bstr_t strReader("");
        if(strReaderName != NULL)
        {
           strReader.Assign(strReaderName);
        }
        tStatus = BEID_Init(strReader, lOCSP, lCRL, &m_lCardHandle);
        FillStatus(&tStatus, pIRetStatus);
        // Store the returned handle
        *plHandle = m_lCardHandle;

        // Make sure a copy is returned
        pIRetStatus.QueryInterface (ppRetStatus);
    }
    return hRes;
}

STDMETHODIMP CEIDlib::Exit(IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    BEID_Status tStatus = {0};
    HRESULT hRes = S_OK;

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Create a local return status object
    if (SUCCEEDED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
    {
        // Exit the eidlib
        tStatus = BEID_Exit();
        FillStatus(&tStatus, pIRetStatus);
        m_lCardHandle = NULL;

        // Make sure a copy is returned
        pIRetStatus.QueryInterface (ppRetStatus);
    }
    return hRes;
}

STDMETHODIMP CEIDlib::GetID(IMapCollection **ppMapCollection, ICertifCheck **ppCertifCheck, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    CComPtr <IMapCollection> pIMapCollection;
    CComPtr <ICertifCheck> pICertifCheck;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(ppMapCollection == NULL || *ppMapCollection != NULL)
        return E_POINTER;
    if(ppCertifCheck == NULL || *ppCertifCheck != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;
   // Create a map collection to contain the ID data
   if (FAILED(hRes = pIMapCollection.CoCreateInstance (CLSID_MapCollection)))
        return hRes;
   // Create a CertifCheck object
   if (FAILED(hRes = pICertifCheck.CoCreateInstance (CLSID_CertifCheck)))
        return hRes;

    BEID_ID_Data tIDData = {0};
    BEID_Certif_Check tCertifCheck = {0};

    // Get the ID data
    tStatus = BEID_GetID(&tIDData, &tCertifCheck);
    FillStatus(&tStatus, pIRetStatus);

    if(BEID_OK == tStatus.general)
    {
        // Copy the data into the map that will be returned
        pIMapCollection->SetValue (_bstr_t("Version"), _variant_t((short)tIDData.version));
        pIMapCollection->SetValue (_bstr_t("CardNumber"), _variant_t(tIDData.cardNumber));
        pIMapCollection->SetValue (_bstr_t("ChipNumber"), _variant_t(tIDData.chipNumber));
        pIMapCollection->SetValue (_bstr_t("BeginValidityDate"), _variant_t(tIDData.validityDateBegin));
        pIMapCollection->SetValue (_bstr_t("EndValidityDate"), _variant_t(tIDData.validityDateEnd));
        pIMapCollection->SetValue (_bstr_t("IssuingMunicipality"), _variant_t(tIDData.municipality));
        pIMapCollection->SetValue (_bstr_t("NationalNumber"), _variant_t(tIDData.nationalNumber));
        pIMapCollection->SetValue (_bstr_t("Name"), _variant_t(tIDData.name));
        pIMapCollection->SetValue (_bstr_t("FirstName1"), _variant_t(tIDData.firstName1));
        pIMapCollection->SetValue (_bstr_t("FirstName2"), _variant_t(tIDData.firstName2));
        pIMapCollection->SetValue (_bstr_t("FirstName3"), _variant_t(tIDData.firstName3));
        pIMapCollection->SetValue (_bstr_t("Nationality"), _variant_t(tIDData.nationality));
        pIMapCollection->SetValue (_bstr_t("BirthPlace"), _variant_t(tIDData.birthLocation));
        pIMapCollection->SetValue (_bstr_t("BirthDate"), _variant_t(tIDData.birthDate));
        pIMapCollection->SetValue (_bstr_t("Gender"), _variant_t(tIDData.sex));
        pIMapCollection->SetValue (_bstr_t("NobilityTitle"), _variant_t(tIDData.nobleCondition));
        pIMapCollection->SetValue (_bstr_t("DocumentType"), _variant_t((long)tIDData.documentType));
        pIMapCollection->SetValue (_bstr_t("WhiteCane"), _variant_t(tIDData.whiteCane ? true : false));
        pIMapCollection->SetValue (_bstr_t("YellowCane"), _variant_t(tIDData.yellowCane ? true : false));
        pIMapCollection->SetValue (_bstr_t("ExtendedMinority"), _variant_t(tIDData.extendedMinority ? true : false));

        CComSafeArray safeArray;
        safeArray.CreateOneDim(VT_UI1, BEID_MAX_HASH_PICTURE_LEN, tIDData.hashPhoto);
        pIMapCollection->SetValue (_bstr_t("HashPhoto"), safeArray);
        safeArray.Clear(); 

        // Return the map
        pIMapCollection.QueryInterface (ppMapCollection);

        // Copy CertifData
        FillCertifCheck(&tCertifCheck, pICertifCheck);
        pICertifCheck.QueryInterface (ppCertifCheck);
    }

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

    return hRes;
}

STDMETHODIMP CEIDlib::GetAddress(IMapCollection **ppMapCollection, ICertifCheck **ppCertifCheck, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    CComPtr <IMapCollection> pIMapCollection;
    CComPtr <ICertifCheck> pICertifCheck;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(ppMapCollection == NULL || *ppMapCollection != NULL)
        return E_POINTER;
    if(ppCertifCheck == NULL || *ppCertifCheck != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;
   // Create a map collection to contain the ID data
   if (FAILED(hRes = pIMapCollection.CoCreateInstance (CLSID_MapCollection)))
        return hRes;
   // Create a CertifCheck object
   if (FAILED(hRes = pICertifCheck.CoCreateInstance (CLSID_CertifCheck)))
        return hRes;

    BEID_Address tAddress = {0};
    BEID_Certif_Check tCertifCheck = {0};

    // Get the Address data
    tStatus = BEID_GetAddress(&tAddress, &tCertifCheck);
    FillStatus(&tStatus, pIRetStatus);    

    if(BEID_OK == tStatus.general)
    {
        // Copy the data into the map that will be returned
        pIMapCollection->SetValue (_bstr_t("Version"), _variant_t((short)tAddress.version));
        pIMapCollection->SetValue (_bstr_t("Street"), _variant_t(tAddress.street));
        pIMapCollection->SetValue (_bstr_t("HouseNumber"), _variant_t(tAddress.streetNumber));
        pIMapCollection->SetValue (_bstr_t("BoxNumber"), _variant_t(tAddress.boxNumber));
        pIMapCollection->SetValue (_bstr_t("ZIPCode"), _variant_t(tAddress.zip));
        pIMapCollection->SetValue (_bstr_t("Municipality"), _variant_t(tAddress.municipality));
        pIMapCollection->SetValue (_bstr_t("Country"), _variant_t(tAddress.country));
    
        // Return the map
        pIMapCollection.QueryInterface (ppMapCollection);

        // Copy CertifData
        FillCertifCheck(&tCertifCheck, pICertifCheck);
        pICertifCheck.QueryInterface (ppCertifCheck);
    }

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::GetPicture(IMapCollection **ppMapCollection, ICertifCheck **ppCertifCheck, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    CComPtr <IMapCollection> pIMapCollection;
    CComPtr <ICertifCheck> pICertifCheck;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(ppMapCollection == NULL || *ppMapCollection != NULL)
        return E_POINTER;
    if(ppCertifCheck == NULL || *ppCertifCheck != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;
   // Create a map collection to contain the ID data
   if (FAILED(hRes = pIMapCollection.CoCreateInstance (CLSID_MapCollection)))
        return hRes;
   // Create a CertifCheck object
   if (FAILED(hRes = pICertifCheck.CoCreateInstance (CLSID_CertifCheck)))
        return hRes;

    BEID_Bytes tPicture = {0};
    BYTE buffer[4096] = {0};
    tPicture.length = 4096;
    tPicture.data = buffer;

    BEID_Certif_Check tCertifCheck = {0};
    // Get the picture data
    tStatus = BEID_GetPicture(&tPicture, &tCertifCheck);
    FillStatus(&tStatus, pIRetStatus);    

    if(BEID_OK == tStatus.general)
    {
        // Copy the data into the map that will be returned
        CComSafeArray safeArray;
        safeArray.CreateOneDim(VT_UI1, tPicture.length, tPicture.data);
        pIMapCollection->SetValue (_bstr_t("Picture"), safeArray);
        safeArray.Clear();

        // Return the map
        pIMapCollection.QueryInterface (ppMapCollection);
        // Copy CertifData
        FillCertifCheck(&tCertifCheck, pICertifCheck);
        pICertifCheck.QueryInterface (ppCertifCheck);
    }

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::GetVersionInfo(BOOL bSignature, IMapCollection **ppMapCollection, VARIANT *pvtSignature, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    CComPtr <IMapCollection> pIMapCollection;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(ppMapCollection == NULL || *ppMapCollection != NULL)
        return E_POINTER;
    if(pvtSignature == NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;
    // Create a map collection to contain the ID data
    if (FAILED(hRes = pIMapCollection.CoCreateInstance (CLSID_MapCollection)))
        return hRes;

    // Get the Version Info
    BEID_VersionInfo tInfo = {0};
    BEID_Bytes tBytes = {0};
    tStatus = BEID_GetVersionInfo(&tInfo, bSignature, &tBytes);
    FillStatus(&tStatus, pIRetStatus);

    if(BEID_OK == tStatus.general)
    {
        // Copy the data into the map that will be returned
        CComSafeArray safeArraySerial;
        safeArraySerial.CreateOneDim(VT_UI1, 16, tInfo.SerialNumber);
        pIMapCollection->SetValue (_bstr_t("SerialNumber"), safeArraySerial);
        safeArraySerial.Clear();
        pIMapCollection->SetValue (_bstr_t("ComponentCode"), _variant_t(tInfo.ComponentCode));
        pIMapCollection->SetValue (_bstr_t("OSNumber"), _variant_t(tInfo.OSNumber));
        pIMapCollection->SetValue (_bstr_t("OSVersion"), _variant_t(tInfo.OSVersion));
        pIMapCollection->SetValue (_bstr_t("SoftmaskNumber"), _variant_t(tInfo.SoftmaskNumber));
        pIMapCollection->SetValue (_bstr_t("SoftmaskVersion"), _variant_t(tInfo.SoftmaskVersion));
        pIMapCollection->SetValue (_bstr_t("AppletVersion"), _variant_t(tInfo.AppletVersion));
        pIMapCollection->SetValue (_bstr_t("GlobalOSVersion"), _variant_t((short)tInfo.GlobalOSVersion));
        pIMapCollection->SetValue (_bstr_t("AppletInterfaceVersion"), _variant_t(tInfo.AppletInterfaceVersion));
        pIMapCollection->SetValue (_bstr_t("PKCS1Support"), _variant_t(tInfo.PKCS1Support));
        pIMapCollection->SetValue (_bstr_t("KeyExchangeVersion"), _variant_t(tInfo.KeyExchangeVersion));
        pIMapCollection->SetValue (_bstr_t("ApplicationLifeCycle"), _variant_t(tInfo.ApplicationLifeCycle));
        pIMapCollection->SetValue (_bstr_t("GraphPerso"), _variant_t(tInfo.GraphPerso));
        pIMapCollection->SetValue (_bstr_t("ElecPerso"), _variant_t(tInfo.ElecPerso));
        pIMapCollection->SetValue (_bstr_t("ElecPersoInterface"), _variant_t(tInfo.ElecPersoInterface));
        pIMapCollection->SetValue (_bstr_t("Reserved"), _variant_t(tInfo.Reserved));

        // Return the map
        pIMapCollection.QueryInterface (ppMapCollection);

        // Fill signature
        _variant_t vtSignature;
        if(bSignature && tBytes.length > 0)
        {
            CComSafeArray safeArray;
            safeArray.CreateOneDim(VT_UI1, tBytes.length, tBytes.data);
            vtSignature = safeArray;
            safeArray.Clear(); 
        }
        ::VariantCopy(pvtSignature, &vtSignature);
    }    

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::BeginTransaction(IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    tStatus = BEID_BeginTransaction();
    FillStatus(&tStatus, pIRetStatus);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

    return hRes;
}

STDMETHODIMP CEIDlib::EndTransaction(IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    tStatus = BEID_EndTransaction();
    FillStatus(&tStatus, pIRetStatus);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::FlushCache(IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    tStatus = BEID_FlushCache();
    FillStatus(&tStatus, pIRetStatus);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::SelectApplication(VARIANT vtApplication, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;

    // Make sure it is an array of bytes
	if ((vtApplication.vt & (VT_ARRAY | VT_UI1)) == 0) 
		return E_INVALIDARG;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    BEID_Bytes tBytes = {0};
    CComSafeArray safeArrayApp(vtApplication);
    BYTE *pByteArray = NULL;
	safeArrayApp.AccessData((void **)&pByteArray);
    tBytes.length = safeArrayApp.GetOneDimSize();
    memcpy(tBytes.data, pByteArray, tBytes.length);
    safeArrayApp.UnaccessData();
    
    tStatus = BEID_SelectApplication(&tBytes);
    FillStatus(&tStatus, pIRetStatus);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::SendAPDU(VARIANT vtCommand, IPin *pPIN, VARIANT *pvtResponse, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;
    if(pvtResponse == NULL)
        return E_POINTER;

    // Make sure it is an array of bytes
	if ((vtCommand.vt & (VT_ARRAY | VT_UI1)) == 0) 
		return E_INVALIDARG;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    BEID_Bytes tBytesCmd = {0};
    BEID_Bytes tBytesResp = {0};
    // Fill command
    CComSafeArray safeArrayCmd(vtCommand);
    BYTE *pByteArray = NULL;
	safeArrayCmd.AccessData((void **)&pByteArray);
    tBytesCmd.length = safeArrayCmd.GetOneDimSize();
    memcpy(tBytesCmd.data, pByteArray, tBytesCmd.length);
    safeArrayCmd.UnaccessData();

    // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);

    // Send APDU
    tStatus = BEID_SendAPDU(&tBytesCmd, &tPin, &tBytesResp);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Fill response
    CComSafeArray safeArrayResp;
    safeArrayResp.CreateOneDim(VT_UI1, tBytesResp.length, tBytesResp.data); 
    ::VariantCopy(pvtResponse, safeArrayResp);
    safeArrayResp.Clear(); 

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::VerifyPIN(IPin *pPIN, BSTR bstrPin, long *plTriesLeft, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;
    if(plTriesLeft == NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);
    _bstr_t strPin(bstrPin, true);

    // VerifyPIN
    tStatus = BEID_VerifyPIN(&tPin, strPin.length() > 0 ? (char *)strPin : NULL, plTriesLeft);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::ChangePin(IPin *pPIN, BSTR strOldPin, BSTR strNewPin, long *plTriesLeft, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;
    if(plTriesLeft == NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

     // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);
    _bstr_t strPinOld(strOldPin, true);
    _bstr_t strPinNew(strNewPin, true);

    // VerifyPIN
    tStatus = BEID_ChangePIN(&tPin, strPinOld.length() > 0 ? (char *)strPinOld : NULL, strPinNew.length() > 0 ? (char *)strPinNew : NULL, plTriesLeft);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::GetPinStatus(IPin *pPIN, BOOL bSignature, VARIANT *pvtSignature, long *plTriesLeft, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;
    if(plTriesLeft == NULL)
        return E_POINTER;
    if(pvtSignature == NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

     // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);

    // GetPINStatus
    BEID_Bytes tBytes = {0};    
    tStatus = BEID_GetPINStatus(&tPin, plTriesLeft, bSignature, &tBytes);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Fill signature
    _variant_t vtSignature;
    if(bSignature && tBytes.length > 0)
    {
        CComSafeArray safeArray;
        safeArray.CreateOneDim(VT_UI1, tBytes.length, tBytes.data);
        vtSignature = safeArray;
        safeArray.Clear();
    }
    ::VariantCopy(pvtSignature, &vtSignature);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::ReadFile(IPin *pPIN, VARIANT vtFileId, VARIANT *pvtData, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;
    if(pvtData == NULL)
        return E_POINTER;

    // Make sure it is an array of bytes
	if ((vtFileId.vt & (VT_ARRAY | VT_UI1)) == 0) 
		return E_INVALIDARG;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

     // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);

    // ReadFile
    BEID_Bytes tBytesFile = {0};
    CComSafeArray safeArrayFile(vtFileId);
    BYTE *pByteArray = NULL;
	safeArrayFile.AccessData((void **)&pByteArray);
    tBytesFile.length = safeArrayFile.GetOneDimSize();
    memcpy(tBytesFile.data, pByteArray, tBytesFile.length);
    safeArrayFile.UnaccessData();
    
    BEID_Bytes tBytesRet = {0};    
    tStatus = BEID_ReadFile(&tBytesFile, &tBytesRet, &tPin);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Fill response
    CComSafeArray safeArrayResp;
    if(BEID_OK == tStatus.general && tBytesRet.length > 0)
    {
        safeArrayResp.CreateOneDim(VT_UI1, tBytesRet.length, tBytesRet.data); 
    }
    ::VariantCopy(pvtData, safeArrayResp);
    safeArrayResp.Clear();

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::WriteFile(IPin *pPIN, VARIANT vtFileID, VARIANT vtData, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pPIN == NULL)
        return E_POINTER;

    // Make sure it is an array of bytes
	if ((vtFileID.vt & (VT_ARRAY | VT_UI1)) == 0) 
		return E_INVALIDARG;
	if ((vtData.vt & (VT_ARRAY | VT_UI1)) == 0) 
		return E_INVALIDARG;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

     // Fill PinData
    BEID_Pin tPin = {0};
    FillPinData(&tPin, pPIN);

    // WriteFile
    BEID_Bytes tBytesFile = {0};
    CComSafeArray safeArrayFile(vtFileID);
    BYTE *pByteArray = NULL;
	safeArrayFile.AccessData((void **)&pByteArray);
    tBytesFile.length = safeArrayFile.GetOneDimSize();
    memcpy(tBytesFile.data, pByteArray, tBytesFile.length);
    safeArrayFile.UnaccessData();
    
    BEID_Bytes tBytesWrite = {0};
    CComSafeArray safeArrayWrite(vtData);
    BYTE *pByteArrayWrite = NULL;
	safeArrayWrite.AccessData((void **)&pByteArrayWrite);
    tBytesWrite.length = safeArrayWrite.GetOneDimSize();
    memcpy(tBytesWrite.data, pByteArrayWrite, tBytesWrite.length);
    safeArrayWrite.UnaccessData();

    tStatus = BEID_WriteFile(&tBytesFile, &tBytesWrite, &tPin);
    FillStatus(&tStatus, pIRetStatus);

    DeletePinData(&tPin);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}


/////////////////////////
// Helper Functions
////////////////////////

void CEIDlib::FillStatus(BEID_Status *pStatus, IRetStatus *pRetStatus)
{
    if(NULL != pStatus && NULL != pRetStatus)
    {
        pRetStatus->SetGeneral(pStatus->general);
        pRetStatus->SetSystem(pStatus->system);
        pRetStatus->SetPCSC(pStatus->pcsc);
        CComSafeArray safeArray;
        safeArray.CreateOneDim(VT_UI1, 2, pStatus->cardSW);
        pRetStatus->SetCardSW(safeArray);
        safeArray.Clear(); 
    }
}

void CEIDlib::FillCertifCheck(BEID_Certif_Check *pCheck, ICertifCheck *pRetCheck)
{
    if(NULL != pCheck && NULL != pRetCheck)
    {
        CComPtr<ICertif> pICertif;
        CComSafeArray safeArrayCertifs;
        CComSafeArray safeArrayCertif;
        pRetCheck->SetSignatureCheck(pCheck->signatureCheck);
        pRetCheck->SetPolicy(pCheck->usedPolicy);
        safeArrayCertifs.CreateOneDim(VT_DISPATCH, pCheck->certificatesLength);

        for(int i = 0; i < pCheck->certificatesLength; ++i)
        {
            if(SUCCEEDED(pICertif.CoCreateInstance(CLSID_Certif)))
            {
                pICertif->SetStatus(pCheck->certificates[i].certifStatus);
                pICertif->SetLabel(_bstr_t(pCheck->certificates[i].certifLabel));
                safeArrayCertif.CreateOneDim(VT_UI1,  pCheck->certificates[i].certifLength, pCheck->certificates[i].certif);
                pICertif->SetCertif(safeArrayCertif);
                safeArrayCertif.Clear(); 
                safeArrayCertifs.PutElement(i, pICertif);
                pICertif.Release();
            }
        }
        pRetCheck->SetCertificates(safeArrayCertifs);
        safeArrayCertifs.Clear(); 
    }
}

void CEIDlib::FillPinData(BEID_Pin *pPin, IPin *pIPin)
{
    if(NULL != pPin && NULL != pIPin)
    {
        _variant_t vtID;
        int iLen = 0;
        BSTR bstrShortUsage;
        BSTR bstrLongUsage;
        pIPin->GetPinType(&(pPin->pinType));
        pIPin->GetID(&vtID);
        pPin->id = (BYTE)vtID;
        pIPin->GetUsageCode((&pPin->usageCode));
        pIPin->GetShortUsage(&bstrShortUsage);
        _bstr_t strShortUsage(bstrShortUsage, false);
        iLen = strShortUsage.length();
        if(iLen > 0)
        {
            pPin->shortUsage = new char[iLen+ 1];
            memset(pPin->shortUsage, 0, iLen+1);
            strcpy(pPin->shortUsage, (char *)strShortUsage);
        }
        else
        {
            pPin->shortUsage = NULL;
        }
        pIPin->GetLongUsage(&bstrLongUsage);
        _bstr_t strLongUsage(bstrLongUsage, false);
        iLen = strLongUsage.length();
        if(iLen > 0)
        {
            pPin->longUsage = new char[iLen+ 1];
            memset(pPin->longUsage, 0, iLen+1);
            strcpy(pPin->longUsage, (char *)strLongUsage);
        }
        else
        {
            pPin->longUsage = NULL;
        }
    }
}

void CEIDlib::DeletePinData(BEID_Pin *pPin)
{
    if(NULL != pPin)
    {
        if(NULL != pPin->shortUsage)
        {
            delete [] pPin->shortUsage;
            pPin->shortUsage = NULL;
        }
        if(NULL != pPin->longUsage)
        {
            delete [] pPin->longUsage;
            pPin->longUsage = NULL;
        }
    }
}

STDMETHODIMP CEIDlib::SetRawData(IRaw *pRaw, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(pRaw == NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    BYTE *pByteArray = NULL;
    unsigned long ulLen = 0;
    BEID_Raw tRaw = {0};
    _variant_t vtData;
    CComSafeArray safeArray;

    pRaw->GetIDData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.idData, pByteArray, ulLen);
        tRaw.idLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetIDSigData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.idSigData, pByteArray, ulLen);
        tRaw.idSigLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetAddrData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.addrData, pByteArray, ulLen);
        tRaw.addrLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetAddrSigData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.addrSigData, pByteArray, ulLen);
        tRaw.addrSigLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetPictureData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.pictureData, pByteArray, ulLen);
        tRaw.pictureLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetCardData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.cardData, pByteArray, ulLen);
        tRaw.cardDataLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetTokenInfoData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.tokenInfo, pByteArray, ulLen);
        tRaw.tokenInfoLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetRNData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.certRN, pByteArray, ulLen);
        tRaw.certRNLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetChallengeData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.challenge, pByteArray, ulLen);
        tRaw.challengeLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    pRaw->GetResponseData(&vtData);
    if(vtData.vt & VT_ARRAY)
    {
        safeArray = vtData;
        safeArray.AccessData((void **)&pByteArray);
        ulLen = safeArray.GetOneDimSize();
        memcpy(tRaw.response, pByteArray, ulLen);
        tRaw.responseLength = ulLen;
        safeArray.UnaccessData();
        ulLen = 0;
        pByteArray = NULL;
        safeArray.Clear();
    }

    tStatus = BEID_SetRawData(&tRaw);
    FillStatus(&tStatus, pIRetStatus);

    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

STDMETHODIMP CEIDlib::GetRawData(IRaw **ppRaw, IRetStatus **ppRetStatus)
{
    CComPtr <IRetStatus> pIRetStatus;
    CComPtr <IRaw> pIRaw;
    HRESULT hRes = S_OK;
    BEID_Status tStatus = {0};

    if(ppRetStatus == NULL || *ppRetStatus != NULL)
        return E_POINTER;
    if(ppRaw == NULL || *ppRaw != NULL)
        return E_POINTER;

    // Create a local return status object
    if (FAILED(hRes = pIRetStatus.CoCreateInstance (CLSID_RetStatus)))
        return hRes;

    // Create a local return status object
    if (FAILED(hRes = pIRaw.CoCreateInstance (CLSID_Raw)))
        return hRes;

    BEID_Raw tRaw = {0};
    tStatus = BEID_GetRawData(&tRaw);
    FillStatus(&tStatus, pIRetStatus);

    if(BEID_OK == tStatus.general)
    {
        FillRawData(&tRaw, pIRaw);
        pIRaw.QueryInterface (ppRaw);
    }
    // Make sure a copy is returned
    pIRetStatus.QueryInterface (ppRetStatus);

	return hRes;
}

void CEIDlib::FillRawData(BEID_Raw *pRaw, IRaw *pIRaw)
{
    if(NULL != pRaw && NULL != pIRaw)
    {
        CComSafeArray safeArray;
        if(pRaw->idLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->idLength, pRaw->idData);
            pIRaw->SetIDData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->idSigLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->idSigLength, pRaw->idSigData);
            pIRaw->SetIDSigData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->addrLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->addrLength, pRaw->addrData);
            pIRaw->SetAddrData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->addrSigLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->addrSigLength, pRaw->addrSigData);
            pIRaw->SetAddrSigData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->pictureLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->pictureLength, pRaw->pictureData);
            pIRaw->SetPictureData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->certRNLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->certRNLength, pRaw->certRN);
            pIRaw->SetRNData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->cardDataLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->cardDataLength, pRaw->cardData);
            pIRaw->SetCardData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->tokenInfoLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->tokenInfoLength, pRaw->tokenInfo);
            pIRaw->SetTokenInfoData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->challengeLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->challengeLength, pRaw->challenge);
            pIRaw->SetChallengeData(safeArray);
            safeArray.Clear(); 
        }
        if(pRaw->responseLength > 0)
        {
            safeArray.CreateOneDim(VT_UI1,  pRaw->responseLength, pRaw->response);
            pIRaw->SetResponseData(safeArray);
            safeArray.Clear(); 
        }
    }
}