//
// MyPropertySheet.cpp : implementation file
//******************************************************************************
//Author : Jason Gurgel
//EMail  : jmgurgel@attbi.com
//(C)    : 7-16-2002
//
//Purpose: Provide a professional looking property sheet control with a header
//       : for displaying an icon, a title line, and description text
//       : for each tab in the property sheet. 
//		 : Also to provide an option to place the dialog buttons to the right
//       : of the tab ctrl or leave them in the default location.
//       : Also allow the pageup and pagedown keys to move between tabs.
//		  
//Note   : This code was insipired by Mustafa Demirhan's article about hacking
//		 : the CPropterySheet. See link below. 
//		 : http://www.codeproject.com/property/hacking_the_cpropertyshee.asp#xx162207xx
//
//Disclaimer: This code is free to use as long as this information
//			: is included. The author takes no responsibility for
//			: the corretness of the code or any damage incured from it's use. 
//			: It is free so use at your own risk, and enjoy!
//			: Any comments, suggestions, or additions would be appericated.
//******************************************************************************

// Includes
#include "stdafx.h"
#include "resource.h"
#include "MyPropertySheet.h"
#include "MyPropertySheet.h"

static UINT BASED_CODE indicators[] =
{
    ID_INDICATOR_EIDMSG,
    ID_INDICATOR_EIDBLANK
};

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMyPropertySheet

IMPLEMENT_DYNAMIC(CMyPropertySheet, CPropertySheet)

CMyPropertySheet::CMyPropertySheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
	//Init the class for use
	Init();
}

CMyPropertySheet::CMyPropertySheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	//Init the class for use
	Init();
}


CMyPropertySheet::~CMyPropertySheet()
{
}


/////////////////////////////////////////////////////////////////////////////
// Message Map

BEGIN_MESSAGE_MAP(CMyPropertySheet, CPropertySheet)
	//{{AFX_MSG_MAP(CMyPropertySheet)
	ON_BN_CLICKED(IDCANCEL, OnCancel)
	ON_BN_CLICKED(IDOK, OnOK)	
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Operations

void CMyPropertySheet::HideApplyButton(const BOOL bHide)
{
	if (TRUE == bHide)
	{
		m_psh.dwFlags=m_psh.dwFlags | PSH_NOAPPLYNOW;
	}

	else
	{
		m_psh.dwFlags=m_psh.dwFlags &~PSH_NOAPPLYNOW;
	}
}


void CMyPropertySheet::ShowHelpButton(const BOOL bShow)
{
	if (TRUE == bShow)
	{
		m_psh.dwFlags=m_psh.dwFlags | PSH_HASHELP;
	}

	else
	{
		m_psh.dwFlags=m_psh.dwFlags &~PSH_HASHELP;
	}
}

void CMyPropertySheet::SetUsePageKeys(const BOOL bPageKeys)
{
	m_bUsePageKeys=bPageKeys;
}


void CMyPropertySheet::UseHeaderCtrl(const BOOL bUse)
{
	m_bHeaderCtrl=bUse;
}

void CMyPropertySheet::SetButtonsRight(const BOOL bRight)
{
	m_bButtonsRight=bRight;
}

void CMyPropertySheet::SetHeaderHeight(const int height)
{
	m_HeaderCtrl.SetHeaderHeight(height);
}


/////////////////////////////////////////////////////////////////////////////
// Access Functions

PSHEETITEM CMyPropertySheet::GetDefaultPSItem() const
{
	PSHEETITEM psItem;

	psItem.cBackground=GetSysColor(COLOR_3DFACE);
	psItem.hIcon=NULL;
	psItem.sDesc="";
	psItem.sTitle="";
	psItem.uAlign=DT_LEFT;

	return psItem;
}


/////////////////////////////////////////////////////////////////////////////
// Internal Functions

void CMyPropertySheet::MoveBtnsRight()
{
	//Variables
	CRect rectWnd;		//Window rect
    CRect rectBtn;		//Button rect
	int btnOffset=0;	//Button offset
	int btnLeft=0;		//Temp button left pos
	int btnWidth=0;		//Button width
	int btnHeight=0;	//Button height
    
   
    //Get the size of the propertysheet
    GetWindowRect(rectWnd);

	//Get the size of a button
	//Use ok since every sheet will have one
    GetDlgItem(IDOK)->GetWindowRect(rectBtn);
    
	//Compute button dims
    btnWidth=rectBtn.Width();
    btnHeight=rectBtn.Height();
    btnOffset=rectWnd.bottom - rectBtn.bottom;
    btnLeft=rectWnd.right - rectWnd.left;

	//Resize the propertysheet to accomodate buttons on the right
    rectWnd.bottom=rectBtn.top;
    rectWnd.right=rectWnd.right + rectBtn.Width() + btnOffset;
    MoveWindow(rectWnd);
    
	//Move all buttons to the right of the propertysheet
    rectBtn.left = btnLeft;
    rectBtn.right = btnLeft + btnWidth;

	//Move the ok button
    rectBtn.top=btnOffset;
    rectBtn.bottom=rectBtn.top + btnHeight;
	GetDlgItem(IDOK)->MoveWindow(rectBtn);

	//Move the cancel button
	rectBtn.top+=btnOffset + btnHeight;
    rectBtn.bottom=rectBtn.top + btnHeight;
	GetDlgItem(IDCANCEL)->MoveWindow(rectBtn);

	//Move the apply now button if wanted
	if (!(m_psh.dwFlags & PSH_NOAPPLYNOW))
	{
		rectBtn.top+=btnOffset + btnHeight;
		rectBtn.bottom=rectBtn.top + btnHeight;
		GetDlgItem(ID_APPLY_NOW)->MoveWindow(rectBtn);
	}

	//Move the help button if wanted
	if (m_psh.dwFlags & PSH_HASHELP)
	{
		rectBtn.top+=btnOffset + btnHeight;
		rectBtn.bottom=rectBtn.top + btnHeight;
		GetDlgItem(IDHELP)->MoveWindow(rectBtn);
	}
}

void CMyPropertySheet::SetHeaderData()
{
	//Variables
	PSHEETITEM psItem;


	if (m_bHeaderCtrl)
	{
		//Get the current page index
	   int nCurrentPage = GetActiveIndex();

	   //Get the current sheet item
	   psItem=m_aTabItems.GetAt(nCurrentPage);

	   //Set new header ctrl information
	   m_HeaderCtrl.SetHeaderData(psItem.hIcon,psItem.sTitle,psItem.sDesc);

	   //Set the new header alignments
	   m_HeaderCtrl.SetTextAlign(psItem.uAlign);
	   m_HeaderCtrl.SetTextAlign(psItem.uAlign);

	   //Set the new header background color
	   m_HeaderCtrl.SetBackgroundColor(psItem.cBackground);
	}
}

void CMyPropertySheet::Init()
{
	m_bUsePageKeys=TRUE;
	m_bButtonsRight=TRUE;
	m_bHeaderCtrl=TRUE;
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}



/////////////////////////////////////////////////////////////////////////////
// Overrides

void CMyPropertySheet::AddPage(CPropertyPage* pPage,PSHEETITEM& rHeaderItem)
{
	//Add page to property sheet
	CPropertySheet::AddPage(pPage);

	//Add header item to array
	if (m_bHeaderCtrl)
	{
		m_aTabItems.Add(rHeaderItem);
	}
}

void CMyPropertySheet::AddPage(CPropertyPage* pPage)
{
	//Variables
	PSHEETITEM psItem;


	//Add page to property sheet
	CPropertySheet::AddPage(pPage);

	//Add blank item to array
	if (m_bHeaderCtrl)
	{
		if (0 == m_aTabItems.GetSize())
		{
			m_aTabItems.Add(GetDefaultPSItem());
		}

		else
		{
			m_aTabItems.Add(m_aTabItems.GetAt(m_aTabItems.GetSize() - 1));
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// CMyPropertySheet message handlers

BOOL CMyPropertySheet::OnInitDialog() 
{
	BOOL bResult = CPropertySheet::OnInitDialog();	
	// TODO: Add your specialized code here

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

//    CRect rw;
   // GetWindowRect(&rw);

    //SetWindowPos(NULL, 0, 0, 800, 600, SWP_NOMOVE | SWP_NOOWNERZORDER );
    //GetWindowRect(&rw);
	//Variables
    
    int		i=0;
	int		ids[]={IDOK,IDCANCEL,IDHELP,ID_APPLY_NOW};
	HWND	hWnd=NULL;
    CRect	rect;

    GetDlgItem (IDOK)->ShowWindow (SW_HIDE);
    GetDlgItem (IDCANCEL)->ShowWindow (SW_HIDE);
    GetDlgItem (ID_APPLY_NOW)->ShowWindow (SW_HIDE);
    GetDlgItem (IDHELP)->ShowWindow (SW_HIDE);

    CString strControlTxt;

    strControlTxt.LoadString (IDS_DONE);
    GetDlgItem (IDCANCEL)->SetWindowText (strControlTxt);

	//Move propertysheet buttons to right side of screen
	if (m_bButtonsRight)
	{
		MoveBtnsRight();
	}

	//Ensure header height is min size to display icon and text
	m_HeaderCtrl.SetHeaderHeight(m_HeaderCtrl.GetHeaderHeight());

	//Resize tab ctrl and propertysheet to accomodate the header
	if (m_bHeaderCtrl)
	{
		//Create the header ctrl and resize dialog window
		m_HeaderCtrl.Init(this);
    
		//Move the tab ctrl down by header height amount
		hWnd=(HWND)GetTabControl()->m_hWnd;
		ASSERT (hWnd != NULL);
		
		::GetWindowRect(hWnd,&rect);
        POINT ptUpperLeft;
        POINT ptBottomRight;

        ptUpperLeft.x = rect.left;
        ptUpperLeft.y = rect.top;
        ::ScreenToClient (m_hWnd, &ptUpperLeft);

        ptBottomRight.x = rect.right;
        ptBottomRight.y = rect.bottom;
        ::ScreenToClient (m_hWnd, &ptBottomRight);

        rect.left = ptUpperLeft.x;
        rect.top = ptUpperLeft.y;
        rect.bottom = ptBottomRight.y;
        rect.right = ptBottomRight.x;

//		ScreenToClient(&rect);
		::SetWindowPos(hWnd,NULL,rect.left, 
						rect.top + m_HeaderCtrl.GetHeaderHeight(),rect.Width(), 
						rect.Height(),SWP_NOZORDER | SWP_NOACTIVATE);
    

		//Move the first tab page down by header height
		//Rest will be sized to this automaticly
        CPropertyPage *pp = GetPage (0);
		hWnd=(HWND)GetPage(0)->m_hWnd;
		ASSERT(hWnd != NULL);
		::GetWindowRect (hWnd,&rect);
//        POINT ptUpperLeft;
//        POINT ptBottomRight;

        ptUpperLeft.x = rect.left;
        ptUpperLeft.y = rect.top;
        ::ScreenToClient (m_hWnd, &ptUpperLeft);

        ptBottomRight.x = rect.right;
        ptBottomRight.y = rect.bottom;
        ::ScreenToClient (m_hWnd, &ptBottomRight);

        rect.left = ptUpperLeft.x;
        rect.top = ptUpperLeft.y;
        rect.bottom = ptBottomRight.y;
        rect.right = ptBottomRight.x;
//		ScreenToClient(&rect);
		::SetWindowPos(hWnd,NULL,rect.left, 
						rect.top + m_HeaderCtrl.GetHeaderHeight(),rect.Width(),
						rect.Height(),SWP_NOZORDER | SWP_NOACTIVATE);
    
		//Move buttons down by header height
		for (i=0;i < sizeof(ids)/sizeof(int);i++)
		{			
			hWnd=::GetDlgItem(m_hWnd,ids[i]);
			if (hWnd != NULL)
			{
				::GetWindowRect(hWnd,&rect);
//                POINT ptUpperLeft;
//                POINT ptBottomRight;

                ptUpperLeft.x = rect.left;
                ptUpperLeft.y = rect.top;
                ::ScreenToClient (m_hWnd, &ptUpperLeft);

                ptBottomRight.x = rect.right;
                ptBottomRight.y = rect.bottom;
                ::ScreenToClient (m_hWnd, &ptBottomRight);

                rect.left = ptUpperLeft.x;
                rect.top = ptUpperLeft.y;
                rect.bottom = ptBottomRight.y;
                rect.right = ptBottomRight.x;
//				ScreenToClient(&rect);
				::SetWindowPos (hWnd,NULL,rect.left, 
								rect.top + m_HeaderCtrl.GetHeaderHeight(),
								rect.Width(),rect.Height(),
								SWP_NOZORDER | SWP_NOACTIVATE);
			}
		}
	}

	//Show the header data for inital tab
	SetHeaderData();
    
    // Add status bar
    m_bar.Create(this); //We create the status bar
    m_bar.SetIndicators(indicators,1); //Set the number of panes 

    CRect rectBar;
    GetClientRect(&rectBar);
    //Size the two panes
    m_bar.SetPaneInfo(0, ID_INDICATOR_EIDMSG, SBPS_NORMAL, rectBar.Width());      
    //m_bar.SetPaneInfo(1, ID_INDICATOR_EIDBLANK, SBPS_STRETCH , 0);

    m_bar.SetPaneText(0 , _T(""));
    //m_bar.SetPaneText(1, "");

    //This is where we actually draw it on the screen
    RepositionBars(AFX_IDW_CONTROLBAR_FIRST,AFX_IDW_CONTROLBAR_LAST, ID_INDICATOR_EIDBLANK);

    LoadStrings();
    
	//Center the window relative to its parent
    CenterWindow();
	
	return bResult;
}

BOOL CMyPropertySheet::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) 
{
	//Variables
    NMHDR* pnmh = (NMHDR*)lParam;


    //Set header info if on tab change
    if (TCN_SELCHANGE == pnmh->code)
    {
 	   SetHeaderData();
    }
  
	return CPropertySheet::OnNotify(wParam, lParam, pResult);
}


BOOL CMyPropertySheet::PreTranslateMessage(MSG* pMsg) 
{
	// TODO: Add your specialized code here and/or call the base class
	if ((pMsg->message==WM_KEYDOWN) && (pMsg->wParam==VK_NEXT) && (m_bUsePageKeys))
	{
		//Ignore if only one CPropertyPage
		if(GetPageCount() > 1)  
		{
			//If first page active, select last page
		   if(GetActiveIndex()==0)
		   {
			  SetActivePage(GetPageCount() - 1);
		   }

		   //Else select the previous page
		   else
		   {
			  SetActivePage(GetActiveIndex() - 1);  
		   }

		   //Change header data for new tab
		   SetHeaderData();
		}
	}

	if ((pMsg->message==WM_KEYDOWN) && (pMsg->wParam==VK_PRIOR))
	{
		//Ignore if only one CPropertyPage
		if(GetPageCount() > 1)  
		{
			//If last page active, select the first page
		   if(GetActiveIndex()==(GetPageCount() - 1))
		   {
			  SetActivePage(0);
		   }

		   //Else select the next page
		   else
		   {
			  SetActivePage(GetActiveIndex() + 1);
		   }

		   //Change header data for new tab
		   SetHeaderData();
		}
	}

	return CPropertySheet::PreTranslateMessage(pMsg);
}

void CMyPropertySheet::OnCancel() 
{
    this->EndDialog (0);
}

void CMyPropertySheet::OnOK() 
{
    // Reroute this message to the active page
    WPARAM wParam;
    wParam = HIWORD (BN_CLICKED) + LOWORD (IDOK);
    GetActivePage ()->SendMessage (WM_COMMAND, wParam, (LPARAM)GetActivePage ()->GetDlgItem (IDOK)->m_hWnd);
}

void CMyPropertySheet::SetStatusBarText(UINT nID)
{
    CString strTemp;
    strTemp.LoadString(nID);
    m_bar.SetPaneText(0, strTemp); 
}

void CMyPropertySheet::SetStatusBarText(const CString & strMsg)
{
    m_bar.SetPaneText(0, strMsg); 
}

void CMyPropertySheet::LoadStrings()
{
    CString strTemp;
    strTemp.LoadString(IDS_STRING_APP_TITLE);
    SetWindowText(strTemp);

    // Fill Tab Titles
    TCITEM tcItem = {0};
    CTabCtrl *pTab = GetTabControl();
    strTemp.LoadString(IDS_STRING_IDENTITY_DLGTITLE);
    tcItem.mask = TCIF_TEXT;
    tcItem.pszText = (LPTSTR)(LPCTSTR)strTemp;
    pTab->SetItem(0, &tcItem);

    strTemp.LoadString(IDS_STRING_CERTIFICATES_DLGTITLE);
    tcItem.pszText = (LPTSTR)(LPCTSTR)strTemp;
    pTab->SetItem(1, &tcItem);

    strTemp.LoadString(IDS_STRING_CARDDATA_DLGTITLE);
    tcItem.pszText = (LPTSTR)(LPCTSTR)strTemp;
    pTab->SetItem(2, &tcItem);

    strTemp.LoadString(IDS_STRING_OPTIONS_DLGTITLE);
    tcItem.pszText = (LPTSTR)(LPCTSTR)strTemp;
    pTab->SetItem(3, &tcItem);

}
