#if !defined(AFX_MYPROPERTYSHEET_H__C1F4500F_5652_4A66_8045_5A1974A79162__INCLUDED_)
#define AFX_MYPROPERTYSHEET_H__C1F4500F_5652_4A66_8045_5A1974A79162__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// MyPropertySheet.h : header file
//******************************************************************************
//Author : Jason Gurgel
//EMail  : jmgurgel@attbi.com
//(C)    : 7-16-2002
//
//Purpose: Provide a professional looking property sheet control with a header
//       : for displaying an icon, a title line, and description text
//       : for each tab in the property sheet. 
//		 : Also to provide an option to place the dialog buttons to the right
//       : of the tab ctrl or leave them in the default location.
//       : Also allow the pageup and pagedown keys to move between tabs.
//		  
//Note   : This code was insipired by Mustafa Demirhan's article about hacking
//		 : the CPropterySheet. See link below. 
//		 : http://www.codeproject.com/property/hacking_the_cpropertyshee.asp#xx162207xx
//
//Disclaimer: This code is free to use as long as this information
//			: is included. The author takes no responsibility for
//			: the corretness of the code or any damage incured from it's use. 
//			: It is free so use at your own risk, and enjoy!
//			: Any comments, suggestions, or additions would be appericated.
//******************************************************************************



// How to use this class
//==============================================================================
//1. Include the following files in your project:
//         - DialogHeader.h
//         - DialogHeader.cpp
//         - MyPropertySheet.h
//         - MyPropertySheet.cpp
//
//2. Include "MyPropertySheet.h" int every file you want to use the class
//
//3. Replace CPropertySheet with CMyPropertySheet to use the new ctrl
//     This can be a member variable: CMyPropertySheet m_PropSheet;
//     or a local variable: CMyPropertySheet PropSheet;
//
//4. Set Header options by accessing the header ctrl.
//	   e.g. -	PropSheet.GetDialogHeaderCtrl().SetIconOffset(5);
//     SEE the DialogHeader.h file for a description of the options
//
//5. Set the PropertySheet options with the following functions:
//     a. SetBtnsRight(TRUE) - places dialog buttons on right side of sheet
//     b. UseHeaderCtrl(TRUE) - show and use a header ctrl for this sheet
//     c. ShowHelpButton(TRUE) - show the help button on the prop sheet
//     d. HideApplyButton(TURE) - hides the apply button on the prop sheet
//     e. SetUsePageKeys(TRUE) - PGUP/PGDN keys switch the active tab
//
//6. Add your pages to the property sheet ctrl
//     - use the normal AddPage(&page) function to add a page with
//       a default header item(blank).
//     - use the AddPage(&page,psItem) function to add a page and
//       it's header data.
//
//7. Display your property sheet with PropSheet.DoModal();
//
//
//Example of adding pages to the sheet:
/*
	//Variables
	CMyPropertySheet	PropSheet("Phylum Options");
	CPropPageGeneral	PPGeneral;
	CPropPageSort		PPSort;


	//Get defualt propertysheet header item
	PSHEETITEM psItem=PropSheet.GetDefaultPSItem();

	//Set Propertysheet default layout
	PropSheet.UseHeaderCtrl(TRUE);
	PropSheet.SetButtonsRight(TRUE);
	PropSheet.ShowHelpButton(TRUE);
	PropSheet.HideApplyBtn(TRUE);

	//Add general tab
	psItem.sTitle="General";
	psItem.sDesc="Set general program and viewing options.";
	psItem.hIcon=::LoadIcon(AfxGetResourceHandle(),MAKEINTRESOURCE(IDI_HARDDRIVE));
	PropSheet.AddPage(&PPGeneral,psItem);

	//Add sorting tab
	psItem.sTitle="Sorting";
	psItem.sDesc="Set program sorting options.";
	psItem.hIcon=::LoadIcon(AfxGetResourceHandle(),MAKEINTRESOURCE(IDI_SORT1));
	PropSheet.AddPage(&PPSort,psItem);

	//Show dialog
	PropSheet.DoModal();
*/
//===============================================================================

        
// Includes
#include "DialogHeader.h"	//For the header ctrl area
#include "Afxtempl.h"		//For CArray class

// Structs: Contains all memebers of the header that can be changed
//        : without affecting a change in header height.
//        : That is the font sizes and offsets must be set once
//        : and will be used for each PSHEETITEM.
typedef struct tagCPropSheetItem
{
	HICON		hIcon;
	CString		sTitle;
	CString		sDesc;
	COLORREF	cBackground;
	UINT		uAlign;
} PSHEETITEM;


/////////////////////////////////////////////////////////////////////////////
// CMyPropertySheet

class CMyPropertySheet : public CPropertySheet
{
	DECLARE_DYNAMIC(CMyPropertySheet)

// Construction
public:
	CMyPropertySheet(UINT nIDCaption, CWnd* pParentWnd = NULL, UINT iSelectPage = 0);
	CMyPropertySheet(LPCTSTR pszCaption, CWnd* pParentWnd = NULL, UINT iSelectPage = 0);

// Attributes
public:
	void			SetHeaderHeight(const int height=0);

// Operations
public:
	void			HideApplyButton(const BOOL bHide=FALSE);
	void			ShowHelpButton(const BOOL bShow=FALSE);
	void			UseHeaderCtrl(const BOOL bUse=TRUE);
	void			SetButtonsRight(const BOOL bRight=TRUE);
	void			SetUsePageKeys(const BOOL bPageKeys=TRUE);
	CArray<PSHEETITEM,PSHEETITEM&>& GetItemArray() {return m_aTabItems;} 
    void            SetStatusBarText(UINT nID);
    void            SetStatusBarText(const CString & strMsg);
    void            LoadStrings();

// Access
public:
	PSHEETITEM		GetDefaultPSItem() const;
	CDialogHeader&	GetDialogHeaderCtrl() {return m_HeaderCtrl;}

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMyPropertySheet)
	public:
	virtual BOOL OnInitDialog();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);
	//}}AFX_VIRTUAL

// User Overrides
	public:
	void AddPage(CPropertyPage* pPage,PSHEETITEM& rHeaderItem);
	void AddPage(CPropertyPage* pPage);

// Implementation
public:
	virtual ~CMyPropertySheet();

	// Generated message map functions
protected:
	//{{AFX_MSG(CMyPropertySheet)
	virtual void OnCancel();
	virtual void OnOK();
    //}}AFX_MSG
	DECLARE_MESSAGE_MAP()

// Internal Functions
protected:
	void MoveBtnsRight();
	void SetHeaderData();
	void Init();

	// Member Variables
protected:
	BOOL							m_bHeaderCtrl;
	BOOL							m_bButtonsRight;
	BOOL							m_bUsePageKeys;
	CDialogHeader					m_HeaderCtrl;
	CArray<PSHEETITEM,PSHEETITEM&>	m_aTabItems; 
    CStatusBar m_bar;
	HICON m_hIcon;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MYPROPERTYSHEET_H__C1F4500F_5652_4A66_8045_5A1974A79162__INCLUDED_)
